//
// Created by maxine on 7/1/24.
//

#pragma once

#include <cstdint>
#include <format>
#include <vector>
#include "Color4.hpp"

// Gets set to whatever your terminal emulator is configured for.
// This means black can be shown as purple if configured that way.
namespace mcolor
{
    enum class AnsiColor : uint8_t
    {
        RESET = 0,
        BOLD,
        DIM = 2,
        FG_BLACK = 30,
        FG_RED,
        FG_GREEN,
        FG_YELLOW,
        FG_BLUE,
        FG_MAGENTA,
        FG_CYAN,
        FG_WHITE,
        FG_DEFAULT = 39,
        BG_BLACK = 40,
        BG_RED,
        BG_GREEN,
        BG_YELLOW,
        BG_BLUE,
        BG_MAGENTA,
        BG_CYAN,
        BG_WHITE,
        BG_DEFAULT = 49,
        FG_BRIGHT_BLACK = 90,
        FG_BRIGHT_RED,
        FG_BRIGHT_GREEN,
        FG_BRIGHT_YELLOW,
        FG_BRIGHT_BLUE,
        FG_BRIGHT_MAGENTA,
        FG_BRIGHT_CYAN,
        FG_BRIGHT_WHITE = 97,
        BG_BRIGHT_BLACK = 100,
        BG_BRIGHT_RED,
        BG_BRIGHT_GREEN,
        BG_BRIGHT_YELLOW,
        BG_BRIGHT_BLUE,
        BG_BRIGHT_MAGENTA,
        BG_BRIGHT_CYAN,
        BG_BRIGHT_WHITE = 107,
    };
}

namespace mcolor::rgbColors
{
    // Basically default ugly standard UNIX colors for now
    static const Color4 BLACK =             {0,0,0};
    static const Color4 RED =               {170, 0, 0};
    static const Color4 GREEN =             {0, 170, 0};
    static const Color4 YELLOW =            {170, 170, 0};
    static const Color4 BLUE =              {0, 0, 170};
    static const Color4 MAGENTA =           {170, 0, 170};
    static const Color4 CYAN =              {0, 170, 170};
    static const Color4 WHITE =             {170, 170, 170};
    static const Color4 DEFAULT =           {170, 170, 170};

    static const Color4 BRIGHT_BLACK =      {8,5, 85,};
    static const Color4 BRIGHT_RED =        {85, 85, 255};
    static const Color4 BRIGHT_GREEN =      {85, 255, 85};
    static const Color4 BRIGHT_YELLOW =     {85, 255, 255};
    static const Color4 BRIGHT_BLUE =       {255, 85, 85};
    static const Color4 BRIGHT_MAGENTA =    {255, 85, 255};
    static const Color4 BRIGHT_CYAN =       {255, 255, 85};
    static const Color4 BRIGHT_WHITE =      {255, 255, 255};
}

namespace mcolor
{
    std::string toEscapeCode(Color4 c, bool bg=false);

    std::string toEscapeCode(AnsiColor c);

#ifdef WIN32
    void windowsSaneify();
#endif

    void printAnsiColorTable();

    std::vector<uint8_t> createRGBScale(uint8_t startValue = 200, uint8_t increment = 5);

    std::vector<Color4> redScaler(Color4 c);

    std::vector<Color4> greenScaler(Color4 c);

    std::vector<Color4> blueScaler(Color4 c);

    std::vector<Color4> RGBColorScale(Color4 startingColor, std::vector<Color4> (*scaler)(Color4));

    void printRGBScale(std::vector<Color4> cs);

    void printRGBConsoleTest();
}
